/*
 * da9052_eh.h: Header file for Event Handler module for DA9052 PM IC
 *
 * DA9052 has a single IRQ line which can be used to indicate 32 various events.
 * Event Handler (EH) module has following primary functions.
 *
 * (1) To receive and acknowledge interrupts from DA9052.
 * (2) Extract all events form DA9052 event registers after receiving interrupt.
 * (3) To provide a mechanism to register call back for specific events of
 *     their interest, and call these call backs when corresponding event occurs.
 * (4) To retrive, store and provide TSI data whenever requested by TSI module.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * 12/05/2009:	Initial version
 * 19/05/2009:	First unit tested version
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef _DA9052_EH_H
#define _DA9052_EH_H

/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Type Definitions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Constant Definitions                                                     */
/*--------------------------------------------------------------------------*/

#define DA9052_EH_DEVICE_NAME		"da9052_eh"
#define DA9052_EH_DEVICE_MAJOR_NO	252

/* IOCTLs */
#define DA9052_IOCTL_EH_DUMP_TREE	1

/* Platform spcific defines */
#define DA9052_IRQ_PIN		S3C64XX_GPN(9)
#define DA9052_IRQ		S3C_EINT(9)

/* Start and End of GPN register */
#define GPN_REG_START		0x7F008830
#define GPN_REG_END		0x7F008840

/* Offset of GPN data register from GPN start */
#define GPN_DAT_OFFSET		4
#define GPN9			9
#define DA9052_IRQ_PIN_BIT_NO	GPN9


/* Define for all possible events */
#define		DCIN_DET_EVE	0
#define		VBUS_DET_EVE	1
#define		DCIN_REM_EVE	2
#define		VBUS_REM_EVE	3
#define		VDD_LOW_EVE	4
#define		ALARM_EVE	5
#define		SEQ_RDY_EVE	6
#define		COMP_1V2	7
#define		ONKEY_EVE	8
#define		ID_FLOAT_EVE	9
#define		ID_GND_EVE	10
#define		CHG_END_EVE	11
#define		TBAT_EVE	12
#define		ADC_EOM_EVE	13
#define		PEN_DOWN_EVE	14
#define		TSI_READY_EVE	15
#define		GPI0_EVE	16
#define		GPI1_EVE	17
#define		GPI2_EVE	18
#define		GPI3_EVE	19
#define		GPI4_EVE	20
#define		GPI5_EVE	21
#define		GPI6_EVE	22
#define		GPI7_EVE	23
#define		GPI8_EVE	24
#define		GPI9_EVE	25
#define		GPI10_EVE	26
#define		GPI11_EVE	27
#define		GPI12_EVE	28
#define		GPI13_EVE	29
#define		GPI14_EVE	30
#define		GPI15_EVE	31

/* Total number of events */
#define 	EVE_CNT		(GPI15_EVE+1)

/* Priotirites */
#define 	PRIO_0		0
#define 	PRIO_1		1
#define 	PRIO_2		2
#define 	PRIO_3		3
#define 	PRIO_4		4
#define 	PRIO_5		5
#define 	PRIO_6		6
#define 	PRIO_7		7
#define 	PRIO_8		8
#define 	PRIO_9		9
#define 	PRIO_10		10
#define 	PRIO_11		11
#define 	PRIO_12		12
#define 	PRIO_13		13
#define 	PRIO_14		14
#define 	PRIO_15		15
#define 	PRIO_16		16
#define 	PRIO_17		17
#define 	PRIO_18		18
#define 	PRIO_19		19
#define 	PRIO_20		20
#define 	PRIO_21		21
#define 	PRIO_22		22
#define 	PRIO_23		23
#define 	PRIO_24		24
#define 	PRIO_25		25
#define 	PRIO_26		26
#define 	PRIO_27		27
#define 	PRIO_28		28
#define 	PRIO_29		29
#define 	PRIO_30		30
#define 	PRIO_31		31

/* Total number of priorities */
#define 	PRIO_CNT	(PRIO_31+1)

/* Define for mapping events with their priorities */
#define 	PRIO_0_EVE	TSI_READY_EVE
#define 	PRIO_1_EVE	PEN_DOWN_EVE
#define 	PRIO_2_EVE	ONKEY_EVE
#define 	PRIO_3_EVE	VDD_LOW_EVE
#define 	PRIO_4_EVE	ADC_EOM_EVE
#define 	PRIO_5_EVE	ALARM_EVE
#define 	PRIO_6_EVE	SEQ_RDY_EVE
#define 	PRIO_7_EVE	COMP_1V2
#define 	PRIO_8_EVE	ID_FLOAT_EVE
#define 	PRIO_9_EVE	ID_GND_EVE
#define 	PRIO_10_EVE	CHG_END_EVE
#define 	PRIO_11_EVE	DCIN_DET_EVE
#define 	PRIO_12_EVE	VBUS_DET_EVE
#define 	PRIO_13_EVE	DCIN_REM_EVE
#define 	PRIO_14_EVE	VBUS_REM_EVE
#define 	PRIO_15_EVE	TBAT_EVE
#define 	PRIO_16_EVE	GPI0_EVE
#define 	PRIO_17_EVE	GPI1_EVE
#define 	PRIO_18_EVE	GPI2_EVE
#define 	PRIO_19_EVE	GPI3_EVE
#define 	PRIO_20_EVE	GPI4_EVE
#define 	PRIO_21_EVE	GPI5_EVE
#define 	PRIO_22_EVE	GPI6_EVE
#define 	PRIO_23_EVE	GPI7_EVE
#define 	PRIO_24_EVE	GPI8_EVE
#define 	PRIO_25_EVE	GPI9_EVE
#define 	PRIO_26_EVE	GPI10_EVE
#define 	PRIO_27_EVE	GPI11_EVE
#define 	PRIO_28_EVE	GPI12_EVE
#define 	PRIO_29_EVE	GPI13_EVE
#define 	PRIO_30_EVE	GPI14_EVE
#define 	PRIO_31_EVE	GPI15_EVE

/* Error code for register/unregister functions */
#define INVALID_NB	2
#define INVALID_EVE	3

/* State for EH thread */
#define	ACTIVE		0
#define	INACTIVE	1

/* Status of nIRQ line */
#define IRQ_HIGH	0
#define IRQ_LOW		1

/* States for TSI state machine */
#define WAIT_FOR_PEN_DOWN	0
#define WAIT_FOR_SAMPLING	1
#define SAMPLING_ACTIVE		2

/* Number of sample to store in TSI FIFO */
#define TSI_FIFO_SIZE		16

/* Polling Frequency of EH in milliseconds */
#define EH_POLLING_FREQ		1

/* To enable debug output for your module, set this to 1 */
#define		DA9052_EH_DEBUG		0

#undef DA9052_DEBUG
#if DA9052_EH_DEBUG
#define DA9052_DEBUG( fmt, args... ) printk( KERN_CRIT "" fmt, ##args )
#else
#define DA9052_DEBUG( fmt, args... )
#endif

/*--------------------------------------------------------------------------*/
/* Structure Definitions                                                    */
/*--------------------------------------------------------------------------*/
 
/**
 * struct da9052_eh_nb - Notifier block structure
 * @nb_list	: List head for notifier block list
 * @eve_type	: Even type to which this notifier block is attached
 * @call_back	: Function that should be called for this event 
 */
 typedef struct {
	struct list_head nb_list;	/* list_head for nb list */
	u8	eve_type;		/* Event type */
	void (*call_back)(u32);		/* Call back function for this event*/
 }da9052_eh_nb;


/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Inline Functions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* External Functions                                                       */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Other Functions                                                          */
/*--------------------------------------------------------------------------*/
s32 da9052_eh_register_nb(da9052_eh_nb* nb);
s32 da9052_eh_unregister_nb(da9052_eh_nb* nb);
s32 da9052_tsi_start_sampling(void);
s32 da9052_tsi_stop_sampling(void);
s32 da9052_tsi_get_data(da9052_tsi_reg *buf, u8 cnt);

#endif /* _DA9052_SSC_H */
