/*
 * da9052_lib.h: helper function for DA9052 drivers
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (28/04/2009):	Created the first draft
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef _DA9052_LIB_H
#define _DA9052_LIB_H

/*--------------------------------------------------------------------------*/
/* System wide include files                                                */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Type Definitions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Constant Definitions                                                     */
/*--------------------------------------------------------------------------*/

#define set_bits(value, mask)		(value | mask)
#define clear_bits(value, mask)		(value & ~(mask))

#define SUCCESS		0
#define FAILURE		1

#define OPEN 		1
#define CLOSE 		0

#define ENABLE		1
#define DISABLE		0

#define TRUE		1
#define FALSE		0

#define SET		1
#define	RESET		0
#define	CLEAR		0

/*--------------------------------------------------------------------------*/
/* Structure Definitions                                                    */
/*--------------------------------------------------------------------------*/ 
/**
 * struct da9052_ssc_msg - Structure description
 * @reg_addr	: DA9052 Register address
 * @data	: For Write: Data to be written on given register
		  For Read : Data has read from given register
 *
 */
typedef struct {
	u8 data;	/* For write/read data to/from register */
	u8 addr;	/* DA9052 Register Address */	
} da9052_ssc_msg;


/**
  * struct da9052_ts_reg - 
  * This structure holds TSi co-ordinate data in same format as that in
  * TSI control registers. Hence EH will be using this structure to buffer TSI
  * data
  * @x_msb 		8 MSB's of X co-ordinate
  * @y_msb 		8 MSB of Y co-ordinate
  * @z_msb 		8 MSB of Z co-ordinate
  * @lsb 		2 LSB's of X, Y & Z co-ordinates
 */ 
 typedef struct {
 	u8	x_msb;
	u8	y_msb;
	u8	z_msb;
	u8	lsb;
 }da9052_tsi_reg;
 
/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/
extern s32 da9052_ssc_write(da9052_ssc_msg*);
extern s32 da9052_ssc_read(da9052_ssc_msg*);
extern s32 da9052_ssc_write_many(da9052_ssc_msg *sscmsg, s32 cnt);
extern s32 da9052_ssc_read_many(da9052_ssc_msg *sscmsg, s32 cnt);

/*--------------------------------------------------------------------------*/
/* Inline Functions                                                         */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* External Functions                                                       */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Other Functions                                                          */
/*--------------------------------------------------------------------------*/
 
#endif /* _DA9052_LIB_H */
