/*
 * da9052_tsi.h: tsi driver file for DA9052
 *
 * This file declares global functions and defines macros 
 * used for DA9052 TSI driver.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (11/06/2009):	This file holds all the data types & macro definitions 
 *			related to the DA9052 TSI driver.
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef _DA9052_TSI_H
#define _DA9052_TSI_H

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Local Macro Definitions                                                  */
/*--------------------------------------------------------------------------*/
#define TSI_INPUT_DEVICE_OFF		0
#define NUM_INPUT_DEVS			1

#define DA9052_VENDOR_ID		0xAAAA
#define DA9052_PRODUCT_ID		0x5555

#define DA9052_TSI_DEVICE_NAME		"da9052_tsi"
#define DA9052_TSI_INPUT_DEV		DA9052_TSI_DEVICE_NAME

#define DA9052_TSI_MAJOR_NUM		(249)

/* LDO to which TSI_REF is connected, LDO is defined in Power Manager */
#define DA9052_TSI_REF_SOURCE		LDO_9


/* Following macros define TSI control regsiter bit values */
#define DA9052_TCA_AUTO_TSI_ENABLE		(1<<0)
#define DA9052_TCA_PEN_DET_ENABLE		(1<<1)
#define DA9052_TCA_TSI_XP_MODE_ENABLE		(1<<2)

#define DA9052_TCA_TSI_DELAY_0SLOTS		(0<<6)
#define DA9052_TCA_TSI_DELAY_2SLOTS		(2<<6)
#define DA9052_TCA_TSI_DELAY_4SLOTS		(3<<6)

#define DA9052_TCA_TSI_SEL_XPLUS		(1<<0)
#define DA9052_TCA_TSI_SEL_XMINUS		(1<<1)
#define DA9052_TCA_TSI_SEL_YPLUS		(1<<2)
#define DA9052_TCA_TSI_SEL_YMINUS		(1<<3)

#define DA9052_TCA_TSI_MUX_XPLUS_ROUTED_ADCIN7		(0<<4)
#define DA9052_TCA_TSI_MUX_YPLUS_ROUTED_ADCIN7		(1<<4)
#define DA9052_TCA_TSI_MUX_XMINUS_ROUTED_ADCIN7		(2<<4)
#define DA9052_TCA_TSI_MUX_YMINUS_ROUTED_ADCIN7		(3<<4)

#define DA9052_TCA_TSI_MAN_ENABLE		(1<<6)
#define DA9052_TCA_TSI_SET_TSIREF		(0<<7)
#define DA9052_TCA_TSI_SET_XY_REF		(1<<7)

#define DA9052_EVETN_B_E_PEN_DOWN		(1<<6)
#define DA9052_EVENT_B_E_TSI_READY		(1<<7)

#define DA9052_IRQMASK_B_PENDOWN_MASK   	(1<<6)
#define DA9052_IRQMASK_B_TSI_READY_MASK 	(1<<7)

#define X_LSB_SHIFT 	(0)
#define Y_LSB_SHIFT 	(2)
#define Z_LSB_SHIFT 	(4)
#define PEN_DET_SHIFT 	(6)
#define X_MSB_SHIFT	(2)
#define Y_MSB_SHIFT	(2)
#define Z_MSB_SHIFT	(2)
#define  X_LSB_MASK	(11 << X_LSB_SHIFT)
#define  Y_LSB_MASK	(11 << Y_LSB_SHIFT)
#define  Z_LSB_MASK	(11 << Z_LSB_SHIFT)
#define  PEN_DET_MASK	(11 << PEN_DET_SHIFT)

#define MAX_SSC_MSG_SIZE	10

#undef DA9052_DEBUG
#if DA9052_TSI_DEBUG
#define DA9052_DEBUG( fmt, args... ) printk( KERN_CRIT "" fmt, ##args )
#else
#define DA9052_DEBUG( fmt, args... )
#endif

/* IOCTL commands for TSI */
#define DA9052_TSI_SET_SAMPLING_MODE		(1)
#define DA9052_TSI_CONFIG_DELAY		   	(2)
#define DA9052_TSI_CONFIG_SLOT_SKIP	   	(3)
#define DA9052_TSI_MEASURE_MANUAL_DATA	   	(4)
#define DA9052_TSI_SET_CALIB_MATRIX	   	(5)
#define DA9052_TSI_GET_TSI_CONFIG		(6)
#define DA9052_TSI_GET_CALIB_CONFIG		(7)
#define DA9052_TSI_SET_CALIB_CONFIG		(8)
#define DA9052_TSI_MEASURE_SEQ			(9)
#define DA9052_TSI_SET_DEVICE_STATE		(10)

#define DA9052_TSI_CONFIG_PEN_DET		(11)
#define DA9052_TSI_REG_PENDWN_EH	   	(12)
#define DA9052_TSI_UNREG_PENDWN_EH	   	(13)
#define DA9052_TSI_DISABLE_IRQ			(14)
#define DA9052_TSI_ENABLE_IRQ			(15)
#define DA9052_TSI_POWER_CONTROL		(16)
#define DA9052_TSI_SET_MANUAL_AXIS		(17)
#define DA9052_TSI_GET_DEVICE_STATE		(18)
#define DA9052_TSI_SET_GPIO_PINS		(19)


/* Calibration */
/* Number of points over which data was gathered */
#define NUM_CALIB_SAMPLES		3

/* Offsets in calibration array of respective co-ordinates */
#define DISPLAY_DATA_OFFSET		0
#define TS_DATA_OFFSET			1

/* Types of co-ordinate samples, */
/* 2 in this case: display co-ordinates & TS co-ordinates */
#define	NUM_OF_SAMPLE_TYPES			2

/* Maximum value tsi delay can have */
#define MAX_TSI_DELAY			TSI_DELAY_4SLOTS

/* Maximum value tsi skip can have */
#define MAX_TSI_SKIP_SLOT		TSI_SKIP_330SLOTS

/*--------------------------------------------------------------------------*/
/* Type Definitions                                                         */
/*--------------------------------------------------------------------------*/
/* ADC_MODE: Sampling rate for ADC module */
/* ECONOMY_MODE = 1 sample / 10msec */
/* FAST_MODE = 1 sample / msec */
enum ADC_MODE {

	ECONOMY_MODE = 0,
	FAST_MODE = 1
};

/* TSI_DELAY: delay between TSI coordinate measurements, in slots */
enum TSI_DELAY {
	TSI_DELAY_0SLOTS = 0,
	TSI_DELAY_1SLOTS = 1,
	TSI_DELAY_2SLOTS = 2,
	TSI_DELAY_4SLOTS = 3
};

/* TSI_SLOT_SKIP: Number of slots to be skipped between */
/*  			measurements of TSI coordinate sets */
enum TSI_SLOT_SKIP{
	TSI_SKIP_0SLOTS = 0,
	TSI_SKIP_2SLOTS = 1,
	TSI_SKIP_5SLOTS = 2,
	TSI_SKIP_10SLOTS = 3,
	TSI_SKIP_30SLOTS = 4,
	TSI_SKIP_80SLOTS = 5,
	TSI_SKIP_130SLOTS = 6, 
	TSI_SKIP_330SLOTS = 7 
};

/* TSI_MUX_SEL: Input lines to TSI multiplexer. */
enum TSI_MUX_SEL
{
	TSI_MUX_XPLUS	= 0,
	TSI_MUX_YPLUS 	= 1,
	TSI_MUX_XMINUS	= 2,
	TSI_MUX_YMINUS	= 3
};

/* TSI_IRQ: DA9052 interrupts for TSI */
/* TSI_PEN_DWN: Pen down Interrupt. */
/* TSI_DATA_RDY: TSI Coordinates ready to be read Interrupt. */
enum TSI_IRQ{
	TSI_PEN_DWN,
	TSI_DATA_RDY
};

/* TSI_COORDINATE: TSI coordinate axes */
enum TSI_COORDINATE{
	X_COORDINATE,
	Y_COORDINATE,
	Z_COORDINATE
};

/* TSI_MODE: TSI conversion sequence */
/* XYZP_MODE = X, Y & Z coordinates are measured */
/* XP_MODE = Only X coordinate is measured */
enum TSI_MEASURE_SEQ{
	XYZP_MODE,
	XP_MODE
};

/**
 * \enum TSI_STATE
 * 0. TSI_AUTO_MODE 		continuosly measure coordinate data.
 * 1. TSI_MANUAL_COORD_X	Manually Measure coordinate along X axis.
 * 2. TSI_MANUAL_COORD_Y	Manually Measure coordinate along Y axis.
 * 3. TSI_MANUAL_COORD_Z	Manually Measure coordinate along Z axis.
 * 4. TSI_MANUAL_SET		Measure one set of x, y & z coordinates.
 * 5. TSI_IDLE			TSI disabled
 */
enum TSI_STATE {
	TSI_AUTO_MODE,
	TSI_MANUAL_COORD_X,
	TSI_MANUAL_COORD_Y,
	TSI_MANUAL_COORD_Z,
	TSI_MANUAL_SET,
	TSI_IDLE
};

/**
 * \union da9052_tsi_cont_reg - Structure description
 * @var da9052_tsi_cont_a: 	Byte access for TSI control reg A
 * @var tsi_cont_a:		Bitwise access for TSI control reg A
 * 	- \var auto_tsi_en	enable or disable auto TSI Measurement
 *	- \var pen_det_en	enable or disable pen detection mechanism
 *	- \var tsi_mode	 	Automatic measure sequence: XP, XYZP mode
 *	- \var tsi_skip		Delay between Two tsi measurments
 *	- \var tsi_delay	Settling Delay before new measuremnt
 */
 typedef union{
	u8 da9052_tsi_cont_a;
	struct{
		u8 auto_tsi_en:1;
		u8 pen_det_en:1;
		u8 tsi_mode:1;
		u8 tsi_skip:3;
		u8 tsi_delay:2;
	}tsi_cont_a;
	
 }da9052_tsi_cont_reg; 

/**
 * \union da9052_tsi_man_cont_reg - Structure description
 * @var da9052_tsi_cont_b:		Byte access for TSI control reg B
 * @var tsi_cont_b:		Bitwise access for TSI control reg B
 *	- \var tsi_sel_0		 X+ switch open / close
 *	- \var tsi_sel_1		 X- switch open / close
 *	- \var tsi_sel_2		 Y+ switch open / close
 *	- \var tsi_sel_3		 Y- switch open / close
 *	- \var tsi_mux		 XY Routing Select
 *	- \var tsi_man		 Start / stop Man conf
 *	- \var tsi_adc_ref	adc reference selection
 */
typedef union{
	u8 da9052_tsi_cont_b;
	struct{
		u8 tsi_sel_0:1;
		u8 tsi_sel_1:1;
		u8 tsi_sel_2:1;
		u8 tsi_sel_3:1;
		u8 tsi_mux:2;
		u8 tsi_man:1;
		u8 tsi_adc_ref:1;
	}tsi_cont_b;
 }da9052_tsi_man_cont_reg;

 /**
 *\struct da9052_tsi_conf - Structure description
 * \var auto_cont:		DA9052 TSI control Register A
 * \var man_cont:		DA9052 TSI control Register B
 * \var tsi_adc_sample_intervel	tsi sample mode: 0- Ecnomy, 1- SPEED mode
 * \var tsi_status		Enable / Disable
 * \var ldo9_en			Enable / Disable
 * \var ldo9_conf		Power immediate change
 * \var tsi_ready_irq_mask	interrupt mask: 0: irq_enable, 1: irq_disable
 * \var tsi_pendown_irq_mask	interrupt mask: 0: irq_enable, 1: irq_disable
 */
 typedef struct{
	da9052_tsi_cont_reg		auto_cont;
	da9052_tsi_man_cont_reg		man_cont;
	u8 				tsi_adc_sample_intervel:1;
	enum TSI_STATE 			state;
	u8 				ldo9_en:1;
	u8 				ldo9_conf:1;
	u8 				tsi_ready_irq_mask:1;
	u8 				tsi_pendown_irq_mask:1;
 }da9052_tsi_conf;
 
/**
 * \struct da9052_tsi_ioctl_data - Structure description
 * da9052_tsi_conf *tsi_conf	TSI configuaration pointer
 * da9052_tsi_data *tsi_data	return pointer for manually measured data
 * calib_cfg_t *calib		pointer to calibration configuration
 * da9052_tsi_data **calib_arr	array of pointers to 
 *				calibration data.
 * u8	pen_det			pen detection flag.
 * u8	seq			TSI measurement sequence.
 * u8	interval		sampling interval.
 * u8   pwr			State of TSI power supply: ENABLE or DISABLE.
 * enum TSI_MEASURE_SEQ	seq	TSI measurement sequence.
 * enum TSI_DELAY delay		TSI delay info.
 * enum TSI_SLOT_SKIP skip	TSI skip info.
 * enum TSI_IRQ tsi_irq		TSI interrupts id.
 * enum TSI_STATE		TSI measurement state
 * enum TSI_COORDINATE		Coordinate axis along which manual 
 *				measurements are to be made.
 */
 typedef struct {
	da9052_tsi_conf 		*tsi_conf;
	da9052_tsi_data 		*tsi_data;
	calib_cfg_t 			*calib;
	da9052_tsi_data 		*calib_arr;
	u8				pen_det;
	u8				interval;
	u8				pwr;
	enum TSI_MEASURE_SEQ		seq;
	enum TSI_DELAY 			delay;
	enum TSI_SLOT_SKIP 		skip;
	enum TSI_IRQ 			tsi_irq;
	enum TSI_STATE			tsi_state;
	enum TSI_STATE*			get_tsi_state;
	enum TSI_COORDINATE		axis;
} da9052_tsi_ioctl_data;

/**
 * \struct da9052_tsi_reg_fifo - Structure description
 * Description: This structure comprises of elements relevant to data
 *		 FIFO that stores data from EH. This data is in the same 
 *		format as that read from the DA9052 hardware.
 *
 * @lock:	semaphore to serialise FIFO access.
 * @head:	Index of head of the FIFO.
 * @tail:	Index of tail of the FIFO.
 * @data:	array to store data.
 */
typedef struct {
	struct semaphore	lock;
	s32			head;
	s32 			tail;
	da9052_tsi_reg		data[TSI_REG_DATA_BUF_SIZE];
} da9052_tsi_reg_fifo;
 
 /**
 * \struct da9052_tsi_info - Structure description
 * @input_dev:		To Register with Linux infrastructure as input device
 * @completion:		To get the kernel thread completion status
 * @tsi_conf:		Configuration details for da9052 TSI
 * @tsi_calib		Calibration details for TSI
 * @timer			Handler to Timer
 * @tsi_data_poll_interval	Timer interval to poll EH module for data in ms  
 * @tsi_penup_count		Number of zero data counts required to declare the penup.
 * @tsi_zero_data_cnt		Number of times we receive zero data from EH.
 * @pen_dwn_event		TSI Pen Down event status
 * @tsi_rdy_event		TSI Ready Event
 * @pd_nb		EH notifier block to register TSI pen down handler
 * @pd_reg_status	status of Pen down handler registeration.
 */
 typedef struct {
	struct input_dev*	input_devs[NUM_INPUT_DEVS];
	da9052_tsi_conf  	tsi_conf;
	calib_cfg_t*		tsi_calib;
	u32 			tsi_data_poll_interval;
	u32			tsi_penup_count;
	u32			tsi_zero_data_cnt;
	u8			pen_dwn_event;
	u8			tsi_rdy_event;
	da9052_eh_nb 		pd_nb;
	u8			pd_reg_status;	
 }da9052_tsi_info; 
 
 
/*--------------------------------------------------------------------------*/
/* Global Variables                                                         */
/*--------------------------------------------------------------------------*/
da9052_tsi_info gda9052_tsi_info;

/*--------------------------------------------------------------------------*/
/* Inline Functions                                                         */
/*--------------------------------------------------------------------------*/
static inline u8  mask_pendwn_irq(u8 val)
{ 
	return (val |= DA9052_IRQMASKB_MPENDOWN);
}

static inline u8  unmask_pendwn_irq(u8 val)
{ 
	return (val &= ~DA9052_IRQMASKB_MPENDOWN);
}

static inline u8  mask_tsi_rdy_irq(u8 val) 
{
	return (val |=DA9052_IRQMASKB_MTSIREADY);
}

static inline u8  unmask_tsi_rdy_irq(u8 val)
{ 
	return (val &= ~DA9052_IRQMASKB_MTSIREADY);
}

static inline u8  enable_ldo9(u8 val) 
{
	return (val |=DA9052_LDO9_LDO9EN);
}

static inline u8  disable_ldo9(u8 val)
{
	return (val &= ~DA9052_LDO9_LDO9EN);
} 

static inline u8  set_auto_tsi_en(u8 val)
{
	return (val |=DA9052_TSICONTA_AUTOTSIEN);
}

static inline u8  reset_auto_tsi_en(u8 val)
{
	return (val &=~DA9052_TSICONTA_AUTOTSIEN);
}

static inline u8  enable_pen_detect(u8 val)
{
	return (val |=DA9052_TSICONTA_PENDETEN);
}

static inline u8  disable_pen_detect(u8 val) 
{
	return (val &=~DA9052_TSICONTA_PENDETEN);
}

static inline u8  enable_xyzp_mode(u8 val)
{
	return (val &= ~DA9052_TSICONTA_TSIMODE);
}

static inline u8  enable_xp_mode(u8 val)
{
	return(val |= DA9052_TSICONTA_TSIMODE);
}

static inline u8  enable_tsi_manual_mode(u8 val)
{
	return(val |= DA9052_TSICONTB_TSIMAN);
}

static inline u8  disable_tsi_manual_mode(u8 val)
{
	return(val &= ~DA9052_TSICONTB_TSIMAN);
}

static inline u8 tsi_sel_xplus_close(u8 val)
{
	return(val |= DA9052_TSICONTB_TSISEL0);
}

static inline u8 tsi_sel_xplus_open(u8 val)
{
	return(val &= ~DA9052_TSICONTB_TSISEL0);
}

static inline u8 tsi_sel_xminus_close(u8 val)
{
	return(val |= DA9052_TSICONTB_TSISEL1);
}

static inline u8 tsi_sel_xminus_open(u8 val)
{
	return(val &= ~DA9052_TSICONTB_TSISEL1);
}

static inline u8 tsi_sel_yplus_close(u8 val)
{
	return(val |= DA9052_TSICONTB_TSISEL2);
}

static inline u8 tsi_sel_yplus_open(u8 val)
{
	return(val &= ~DA9052_TSICONTB_TSISEL2);
}

static inline u8 tsi_sel_yminus_close(u8 val)
{
	return(val |= DA9052_TSICONTB_TSISEL3);
}

static inline u8 tsi_sel_yminus_open(u8 val)
{
	return(val &= ~DA9052_TSICONTB_TSISEL3);
}

static inline u8 adc_mode_economy_mode(u8 val)
{
	return(val &= ~DA9052_ADCCONT_ADCMODE);
}

static inline u8 adc_mode_fast_mode(u8 val)
{
	return(val |= DA9052_ADCCONT_ADCMODE);
}

/*--------------------------------------------------------------------------*/
/* Other Functions                                                          */
/*--------------------------------------------------------------------------*/
ssize_t da9052_tsi_config_delay(enum TSI_DELAY delay);

ssize_t da9052_tsi_config_measure_seq(enum TSI_MEASURE_SEQ seq);

ssize_t da9052_tsi_config_state(enum TSI_STATE state);

ssize_t da9052_tsi_set_sampling_mode(u8 interval);

ssize_t da9052_tsi_config_skip_slots(enum TSI_SLOT_SKIP skip);

ssize_t da9052_config_tsi_delay(enum TSI_DELAY delay);

ssize_t da9052_tsi_get_manual_measurement(da9052_tsi_data* data_buf);

int da9052_tsi_get_calib_display_point( da9052_tsi_data* display);

enum TSI_STATE da9052_tsi_get_drv_state(void);


/*--------------------------------------------------------------------------*/
/* Compile time error checks                                                */
/*--------------------------------------------------------------------------*/
#if (SAMPLE_CNT_FOR_WIN_REF <= 1)
#error "SAMPLE_CNT_FOR_WIN_REF cannot be less than or equal to 1"
#endif

#if (DEFAULT_TSI_STATE > 5)
#error "DEFAULT_TSI_STATE cannot be assigned value greater than 5"
#endif

#if (TSI_SLOT_SKIP_VALUE > 7)
#error "TSI_SLOT_SKIP_VALUE cannot be assigned value greater than 7"
#endif

#if (TSI_DELAY_VALUE > 3)
#error "TSI_DELAY_VALUE cannot be assigned value greater than 3"
#endif

#if (TSI_MODE_VALUE > 1)
#error "TSI_DELAY_VALUE cannot be assigned value greater than 1"
#endif


#endif /* _DA9052_TSI_H */

