/*
 * da9052_tsi_cfg.h: tsi driver configuration file for DA9052
 *
 * This file defines configurations used for DA9052 TSI driver.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (11/06/2009):	This file has all the configuration options 
 *			supported by DA9052 TSI driver.
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */

 
#ifndef _DA9052_TSI_CFG_H
#define _DA9052_TSI_CFG_H

/*--------------------------------------------------------------------------*/
/* User Configurable parameters 					    */
/*--------------------------------------------------------------------------*/
/* To enable debug output for your module, set this to 1 */
#define		DA9052_TSI_DEBUG	0

/* DEFAULT_TSI_STATE:
 * 0. TSI_AUTO_MODE 		continuosly measure coordinate data.
 * 1. TSI_MANUAL_COORD_X	Manually Measure coordinate along X axis
 * 2. TSI_MANUAL_COORD_Y	Manually Measure coordinate along Y axis
 * 3. TSI_MANUAL_COORD_Z	Manually Measure coordinate along Z axis
 * 4. TSI_MANUAL_SET		Measure one set of x, y & z coordinates
 * 5. TSI_IDLE			TSI disabled
 */
#define AUTO_MODE		0
#define MANUAL_COORD_X		1
#define MANUAL_COORD_Y		2
#define MANUAL_COORD_Z		3
#define MANUAL_SET		4
#define IDLE			5
/* Assign a value to this macro from above defined TSI states */
#define DEFAULT_TSI_STATE		AUTO_MODE

/* da9052 TSI static configuration */
/* TSI slot skip possible configurations are [0, 2, 5, 10, 30, 80, 130, 330]*/
/* the TSI_SLOT_SKIP equal value [0, 1, 2, 3, 4, 5, 6, 7] */
#define TSI_SLOT_SKIP_VALUE		0

/* TSI delay possible configurations are [0, 1, 2, 4] */
/* TSI_DELAY value is [0, 1, 2, 3] */
#define TSI_DELAY_VALUE			3

/* TSI Mode XYZP Mode = 0 and XP mode is 1 */
#define TSI_MODE_VALUE			0

/* 1: Enable average filtering, 0: Disable average filtering */
#define ENABLE_AVERAGE_FILTER		1

/* Default value for size of average filter */
#define DEFAULT_AVERAGE_FILTER_SIZE	3

/* 1: Enable window filtering, 0: Disable window filtering */
#define ENABLE_WINDOW_FILTER		1

/* Window Filter */ 
#define TSI_X_WINDOW_SIZE		50
#define TSI_Y_WINDOW_SIZE		50

/* Number of raw data samples to be considered for */
/* identifying first reference sample for window filter */
/* Note: This macro cannot be assigned value less than or equal to 1 */
#define SAMPLE_CNT_FOR_WIN_REF		3

/* TSI sampling mode Economy mode = 0; Fast mode =1*/
#define TSI_ECONOMY_MODE		0
#define TSI_FAST_MODE			1
/* set default TSI mode from above two modes */
#define DEFAULT_TSI_SAMPLING_MODE	TSI_FAST_MODE

/* TSI penup detecting interval in msec*/
#define TS_PENUP_DETECT_INTERVEL	50

/* Calibartion  Enable(1) / Disable(0) */
#define TSI_USE_CALIBRATION   		1

/* Touchscreen panel dimensions */
#define TS_X_MIN	(0)
#define TS_X_MAX	(1023)
#define TS_Y_MIN	(0)
#define TS_Y_MAX	(1023)

/* Display panel dimensions */
#define DISPLAY_X_MIN	(0)
#define DISPLAY_X_MAX	(1023)
#define DISPLAY_Y_MIN	(0)
#define DISPLAY_Y_MAX	(1023)

/* Use debounce */
#define ENABLE_TSI_DEBOUNCE		0

/* The number of samples to be considered for debouncing */
#define TSI_DEBOUNCE_DATA_CNT		3

/* manual conversion time out period in msec*/
#define MANUAL_CONVERSION_TIME_OUT	5000

/*--------------------------------------------------------------------------*/
/* Internal Static configuration parameters                                 */
/*--------------------------------------------------------------------------*/
#define RELEASE

/* Switch to enable or disable the profiling debug */
#define DA9052_TSI_RAW_DATA_PROFILING		0
#define DA9052_TSI_WIN_FLT_DATA_PROFILING  	0
#define DA9052_TSI_AVG_FLT_DATA_PROFILING 	0
#define DA9052_TSI_CALIB_DATA_PROFILING 	0
#define DA9052_TSI_OS_DATA_PROFILING 		0

/* Enable this macro to print debounced data */
#define DA9052_TSI_PRINT_DEBOUNCED_DATA		0

/* Enable this macro to print value  of last */
/* processed valid window filter coordinate */
#define DA9052_TSI_PRINT_PREVIOUS_DATA		0


#if ENABLE_AVERAGE_FILTER
#define TSI_AVERAGE_FILTER_SIZE		DEFAULT_AVERAGE_FILTER_SIZE
#else
#define TSI_AVERAGE_FILTER_SIZE			1
#endif

/* max number of samples in 1 sec. specific to DA9052. */
#define TSI_FAST_MODE_SAMPLE_CNT		1000
#define TSI_ECO_MODE_SAMPLE_CNT			100

/* The number of samples to be read in one polling cycle */
#define TSI_POLL_SAMPLE_CNT			10

/*polling frequency for reading TSI data in Economy mode / Fast mode in ms*/
#define TSI_FAST_MODE_REG_DATA_PROCESSING_INTERVAL	((1000 / TSI_FAST_MODE_SAMPLE_CNT)* TSI_POLL_SAMPLE_CNT)
#define TSI_ECO_MODE_REG_DATA_PROCESSING_INTERVAL	((1000 / TSI_ECO_MODE_SAMPLE_CNT)* TSI_POLL_SAMPLE_CNT)

/* Macro holds value to be assigned for reg data processing interval in msec */
#if DEFAULT_TSI_SAMPLING_MODE
#define DEFAULT_REG_DATA_PROCESSING_INTERVAL	TSI_FAST_MODE_REG_DATA_PROCESSING_INTERVAL
#else
#define DEFAULT_REG_DATA_PROCESSING_INTERVAL	TSI_ECO_MODE_REG_DATA_PROCESSING_INTERVAL
#endif

/* Size of buffer containing co-ordinates in REG format. */
#define TSI_REG_DATA_BUF_SIZE 	(2 * TSI_POLL_SAMPLE_CNT)

/* interval at which raw data is processed in msec */
#define TSI_FAST_MODE_RAW_DATA_PROCESSING_INTERVAL		((1000 / TSI_FAST_MODE_SAMPLE_CNT) * (TSI_AVERAGE_FILTER_SIZE))
#define TSI_ECO_MODE_RAW_DATA_PROCESSING_INTERVAL		((1000 / TSI_ECO_MODE_SAMPLE_CNT) * (TSI_AVERAGE_FILTER_SIZE))

/* Macro holds value to be assigned for raw data processing interval */
#if DEFAULT_TSI_SAMPLING_MODE
#define DEFAULT_RAW_DATA_PROCESSING_INTERVAL	TSI_FAST_MODE_RAW_DATA_PROCESSING_INTERVAL
#else
#define DEFAULT_RAW_DATA_PROCESSING_INTERVAL	TSI_ECO_MODE_RAW_DATA_PROCESSING_INTERVAL
#endif

/* Size of buffer containing RAW data co-ordinates. */
#define TSI_RAW_DATA_BUF_SIZE 	(TSI_REG_DATA_BUF_SIZE * ( (TSI_AVERAGE_FILTER_SIZE / TSI_POLL_SAMPLE_CNT) + 1))

#endif //#ifndef _DA9052_TSI_CFG_H
