/*
 * da9052_tsi_filter.h: tsi driver file for DA9052
 *
 * This file declares global functions and defines macros 
 * related to filter options of DA9052 TSI driver.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright(c) 2010 Dialog Semiconductor Ltd.
 *
 * History:
 *
 * (11/06/2009):	This file declares global functions and defines macros 
*			related to filter options of DA9052 TSI driver.
 *
 * Best Viewed with TabSize=8 and ColumnWidth=80
 */


#ifndef __TS_FILTER_H__
#define __TS_FILTER_H__

/*
 * Touchscreen filter.
 *
 * 
 */

/*--------------------------------------------------------------------------*/
/* Module specific include files                                            */
/*--------------------------------------------------------------------------*/
#include "../include/da9052_tsi_cfg.h"

/*--------------------------------------------------------------------------*/
/* Local Macro Definitions                                                  */
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
/* Type Definitions                                                         */
/*--------------------------------------------------------------------------*/
/**
 * \struct da9052_tsi_data - Structure description
 * @var x:		X co-ordinate
 * @var y:		Y co-ordinate
 * @var z:		Z co-ordinate
 */
 typedef struct {
 	s16	x;
	s16	y;
	s16	z;
}da9052_tsi_data;

/**
 * \struct da9052_tsi_raw_fifo - Structure description
 * Description: This structure comprises of elements relevant to data
 *		 FIFO that stores data from EH. This data is in the same 
 *		format as that read from the DA9052 hardware.
 *
 * @lock:	semaphore to serialise FIFO access.
 * @head:	Index of head of the FIFO.
 * @tail:	Index of tail of the FIFO.
 * @data:	array to store data.
 */
typedef struct {
	struct semaphore 	lock;
	s32		 	head;
	s32 			tail;
	da9052_tsi_data		data[TSI_RAW_DATA_BUF_SIZE];
}da9052_tsi_raw_fifo;

/**
 * \struct tsi_thread_type - Structure description
 *
 * @pid:	process id of this thread.
 * @state:	state of the thread, ACTIVE or INACTIVE.
 * @notifier:	Thread notifier for the OS.
 */
typedef struct {
	u8 			pid;
	u8 			state;
	struct completion 	notifier;
} tsi_thread_type;

/* State for TSI thread */
#define	ACTIVE		0
#define	INACTIVE	1


/*--------------------------------------------------------------------------*/
/* Global Functions                                                          */
/*--------------------------------------------------------------------------*/
void __init da9052_init_tsi_fifos (void);

void clean_tsi_fifos(void);

u32 get_reg_data_cnt (void);

u32 get_reg_free_space_cnt(void);

void da9052_tsi_process_reg_data(void);

/*--------------------------------------------------------------------------*/
/* Other Functions                                                          */
/*--------------------------------------------------------------------------*/
extern u32 da9052_tsi_get_input_dev(u8 off);


#endif	//__TS_FILTER_H__
